// ====================================================================
// SSclass.h
// Written by:
// Last Modified:
//
// Description:
// 
// ====================================================================
#include <iostream>
#include <fstream>
#include "SSclass.h"
using namespace std;


// ====================================================================
// constructor
// ====================================================================
SSclass::SSclass(string sourceFile)
{
   m_sourceFileName = sourceFile;
   m_dataIsSorted   = false;
   m_dataSize       = -1;      // -1 for unable to open file

   // ---- read in data, if possible
   cout << "DEBUG: Reading data file: [" << m_sourceFileName  << "]...\n";

   // TODO: Should check if file exists... skipping for now

   // open the data.txt file for input
   std::fstream inputFile(m_sourceFileName.c_str(), std::ios_base::in);

   // Read all the numbers... 
   //      keep count of how many 
   //      display them just so we can see what is happening
   int   numCount = 0;
   float number;
   cout << "DEBUG: ";
   while (inputFile >> number)
   {
      cout << number << " ";
      numCount++;
   }
   cout << endl;
   m_dataSize = numCount;

   // Allocate enough memory to hold the entire file 
   // contents in an array of floats
   mp_data = new float[numCount];

   if (NULL != mp_data )  // allocation succeeed
   {
      // Go back to beginning of the file
      inputFile.clear();
      inputFile.seekg(0, ios::beg);

      cout << "DEBUG2: ";
      // Read in all the data into the daa array
      //      Display the data again to verify it was read
      int index = 0;
      while (inputFile >> mp_data[index])
      {
         cout << mp_data[index] << " ";
         index++;
      }
      cout << endl;
   }
   else
   {
      m_dataSize = -2;   // -2 for failed memory allocation
   }

   inputFile.close();
   cout << "DEBUG: ...end reading data file\n\n";
}

// ====================================================================
// Destructor
// ====================================================================
SSclass::~SSclass()
{
   delete [] mp_data;
   mp_data = NULL;
}

// ====================================================================
// DataSize
// ====================================================================
int  SSclass::DataSize()
{
   return m_dataSize;
}

// ====================================================================
// SortData --- Insertion Sort
// ====================================================================
void SSclass::SortData()
{
   float key;
   int i;
   for(int j=1; j < m_dataSize; j++)
   {
      key = mp_data[j];
      i = j - 1;
      while ( (mp_data[i] > key) && (i >= 0) )
      {
         mp_data[i+1] = mp_data[i];
         i--;
      }
      mp_data[i+1] = key;
   }

   m_dataIsSorted = true;
}

// ====================================================================
// FindValue
// ====================================================================
bool SSclass::FindValue(float findMe)
{
   bool retVal = false;
   if (true == m_dataIsSorted)
   {
      retVal = BinaryFind(findMe);
   }
   else
   {
      retVal = LinearFind(findMe);
   }
   return retVal;
}

// ====================================================================
// LinearFind
// ====================================================================
bool SSclass::LinearFind(float findMe)
{
   bool retVal = false;
   int i = 0;
   while ((i < m_dataSize) && (false == retVal))
   {
      if (mp_data[i] == findMe) { retVal = true; }
      i++;
   }
   return retVal;
}

// ====================================================================
// BinaryFind
// ====================================================================
bool SSclass::BinaryFind(float findMe)
{
   int left = 0;
   int right = m_dataSize - 1;

   bool found = false;
   int i;

   while ((!found) && (left != right))
   {
      i = left + (right - left)/2;       // truncates decimal part

      cout << "   DEBUG:   checking index = " << i << endl;

      if(findMe == mp_data[i])        { found = true;  }
      else if (findMe < mp_data[i])   { right = i;	  }
      else                            { left  = i+1;   }
   }
   return found;
}

// ====================================================================
// friend operator <<
// ====================================================================
ostream& operator<<(ostream& ostr, const SSclass &ssc)
{
   ostr << ssc.m_sourceFileName << ":\n";

   for (int i=0; i < ssc.m_dataSize; i++)
   {
      ostr << ssc.mp_data[i] << " ";
      if ((i+1) % 5 == 0) { ostr << endl; }
   }
   ostr << endl;
   ostr << "EOD\n";

   return ostr;
}

// ====================================================================
// ====================================================================
// end of file
